@@ -14,6 +14,7 @@ from PyQt4.QtCore import *
 import Utils
 import os
 import time
+import dbus
 
 APP_KEY="475214689"
 APP_SECRET="f9b9d4850089e3b95a5c3e6e1d17bed5"
@@ -31,6 +32,9 @@ class UpdaterSina(QThread):
         self.callback=callback
         self.logger.debug("UpdaterSina:__init__")
         self.accountId=accountId
+        self.userid = 0
+        self.users = []
+        self.update_user_follower_pending = False
 
     def basicAuth(self, source, username, password):
         self.auth = BasicAuthHandler(username, password)
@@ -48,7 +52,7 @@ class UpdaterSina(QThread):
         self.auth.setToken(username, password)
         self.api = API(self.auth)
 
-    def initialize(self, accountId,timelineType,sinceId=None,maxId=None,countnum=20,downpic=False, optype = None):
+    def initialize(self, accountId,timelineType,sinceId=None,maxId=None,countnum=20,downpic=False, optype = None, uid=None, screen_name=None):
         self.logger.debug("UpdaterSina:initialize")
         self.stopped = False
         self.accountId=accountId
@@ -62,6 +66,9 @@ class UpdaterSina(QThread):
         self.userinfo = []
         self.unread_counts = {}
         self.optype = optype # for clear unread counts
+        self.uid = uid # for getting user detail info.
+        self.screen_name = screen_name # for getting user detail info.
+        self.update_user_follower_pending = False
 
     def stop(self):
         try:
@@ -81,13 +88,108 @@ class UpdaterSina(QThread):
     def run(self):
         self.logger.debug("run")
         try:
-            self.update_timeline(self.sinceId,self.maxId,self.countnum)
+            if self.update_user_follower_pending != False:
+                self.update_user_follower_pending = False
+                self.update_user_follower_process()
+            else:
+                self.update_timeline(self.sinceId,self.maxId,self.countnum)
         except WeibopError,we:
             self.completed=False
             self.emit(SIGNAL("finished(bool)"),self.completed)
             self.emit(SIGNAL("show_notice(QString,int)"),we.reason,0)
         self.stop()
+        
+    def get_userid(self):
+        ''' get current account userid'''
+        
+        if self.userid == 0:
+            user = self.api.verify_credentials()
+            if user:
+                self.userid = user.id
+            else:
+                print "get_userid failed"
+        return self.userid
 
+        
+    def get_friendship(self, user):
+        #print "get_friendship2me"
+        param={}
+        param["target_id"]=user['id']
+        (source, target) = self.api.show_friendship(**param)
+        #print target.screen_name, target.following, target.followed_by
+
+        friendship = {}
+        #friendship["id"] = target.id
+        if target.id == self.get_userid():
+            friendship["self"] = True
+        else:
+            friendship["self"] = False
+            
+        friendship["following"] = target.following
+        friendship["followed_by"] = target.followed_by  
+            
+        return friendship
+
+    def update_user_follower(self, uid, accountid, downpic, cursor = -1, getFans = False, count = 20):
+        #print "update_user_follower"
+        self.users = []
+        self.uid = uid
+        self.accountId = accountid
+        self.downpic=downpic
+        
+        self.param={}
+        self.param["user_id"]=uid
+        self.param["cursor"]=cursor
+        self.param["count"]=count
+        self.update_user_follower_pending = True
+        self.getFans = getFans
+        
+    def update_user_follower_process(self):
+        #print "update_user_follower_process"
+        users = None
+        if self.getFans:        
+            #print "update_user_follower_process: fans"
+            users = self.api.followers(**self.param)
+        else:
+            #print "update_user_follower_process: friends"
+            users = self.api.friends(**self.param)
+        #print target.screen_name, target.following, target.followed_by
+
+        #print users.next_cursor
+        self.next_cursor = users.next_cursor
+        
+        if users.users != None:
+            for obj in users.users:
+                user = Utils.build_a_user_from_obj(obj, self.accountId)
+                self.users.append(user)
+                #print user
+                
+                status = user['status']
+                
+                # download images.
+                #print str(time.time()) + ": " + "Begin downloading images"
+                if status != None and self.downpic=="True":
+                    thumbnail=status["thumbnail_pic"]
+                    if thumbnail!=None:
+                        filename=thumbnail.split("/")[-1]
+                        picFilename=Utils.THUMBNAIL_DIR+str(self.accountId)+"_"+str(status["id"])+"_"+filename
+                        if os.access(picFilename, os.W_OK) == 0:
+                            picSource=self.downloader.download(thumbnail)
+                            self.writer.write_to(picFilename, picSource)
+                    
+                    avatar=user["profile_image_url"]
+                    if avatar!=None:
+                        picFilename=Utils.AVATAR_DIR+str(self.accountId)+"_"+str(user["id"])
+                        if os.access(picFilename, os.W_OK) == 0:
+                            picSource=self.downloader.download(avatar)
+                            self.writer.write_to(picFilename, picSource)
+
+        if users.next_cursor == 0:
+            hasnext = False
+        else:
+            hasnext = True
+            
+        self.emit(SIGNAL("userlist_finished(bool)"), hasnext)
 
     def update_timeline(self,sinceId=None,maxId=None,countNum=20,pageNum=1):
         #print "update_timeline"
@@ -129,12 +231,32 @@ class UpdaterSina(QThread):
                 param["type"]=2
             elif self.optype == Utils.TIMELINE_TYPE_COMMENTS:    
                 param["type"]=1
+            elif self.optype == Utils.TIMELINE_TYPE_FAVOUR:    
+                param["type"]=4
             timeline = self.api.reset_count(**param)
+        elif self.timelineType==Utils.TIMELINE_TYPE_USER_SHOW:
+            param={}
+            #print "update_timeline, optype", self.optype
+            if self.screen_name == None:
+                param["user_id"]=self.uid
+            else:
+                param["screen_name"]=self.screen_name
+            timeline = self.api.get_user(**param)
+        elif self.timelineType==Utils.TIMELINE_TYPE_USER_TIMELINE:
+            #print "update_timeline, uid", self.uid
+            
+            param["user_id"]=self.uid
+            #param["screen_name"]=self.screen_name
+            timeline = self.api.user_timeline(**param)
 
         #print str(time.time()) + ": " + "After updater update_timeline"
 
         counts = None
-        if self.timelineType != Utils.TIMELINE_TYPE_UNREAD and self.timelineType != Utils.TIMELINE_TYPE_DEUNREAD and timeline!=None:
+        if self.timelineType == Utils.TIMELINE_TYPE_USER_SHOW and timeline!=None:
+            self.user = Utils.build_a_user_from_obj(timeline, self.accountId)
+            self.user["avatar_large"] = self.user["profile_image_url"].replace("/50/", "/180/")
+            #print self.user["avatar_large"],  self.user["profile_image_url"]
+        elif self.timelineType != Utils.TIMELINE_TYPE_UNREAD and self.timelineType != Utils.TIMELINE_TYPE_DEUNREAD and timeline!=None:
             param_id = {}
             param_id["ids"]=""
             tmp_flag = 0
@@ -188,7 +310,8 @@ class UpdaterSina(QThread):
         elif self.timelineType == Utils.TIMELINE_TYPE_UNREAD:  
             self.unread_counts = Utils.build_a_unread_count_from_obj(timeline)
         
-        if self.timelineType==Utils.TIMELINE_TYPE_FRIENDS and param_id["ids"]!="": 
+        if (self.timelineType==Utils.TIMELINE_TYPE_FRIENDS or \
+            self.timelineType==Utils.TIMELINE_TYPE_USER_TIMELINE) and param_id["ids"]!="": 
             counts = self.api.counts(**param_id)
             for _count in counts:
                 tmp_count = Utils.build_a_count_from_obj(_count,self.accountId)
@@ -204,6 +327,9 @@ class UpdaterSina(QThread):
         self.completed = True
         if self.timelineType==Utils.TIMELINE_TYPE_UNREAD:
             self.emit(SIGNAL("unread_finished(int)"),self.accountId)
+        elif self.timelineType==Utils.TIMELINE_TYPE_USER_SHOW:
+            #print self.uid
+            self.emit(SIGNAL("user_finished(QString, QString)"), str(self.uid), str(self.screen_name))
         elif self.timelineType!=Utils.TIMELINE_TYPE_ST_COMMENTS and self.timelineType!=Utils.TIMELINE_TYPE_DEUNREAD:   
             self.emit(SIGNAL("finished(bool)"),self.completed)
         elif self.timelineType == Utils.TIMELINE_TYPE_ST_COMMENTS:
@@ -239,7 +365,7 @@ class SenderSina(QThread):
         self.accountId=accountId
         self.param=param
         self.completed = False
-
+                
     def stop(self):
         try:
             self.mutex.lock()
@@ -299,9 +425,10 @@ class SenderSina(QThread):
                 if param["without_mention"]==2:
                     param1={}
                     param1["id"]=param["id"]
-                    param1["status"]=param["comment"]
+                    param1["status"]=param["comment"]+param["retext"]
                     #print  " param1[status]" + param1["status"] + " id : " + str( param1["id"])
                     param["without_mention"]=0
+                    param["retext"] =None
                     #print "id" + str(param1["id"]) + "   cid" + str(param["cid"])
                     status=self.api.repost(**param1)
                 #print  " param[comment]" + param["comment"] + " id : " + str( param["id"])
@@ -316,4 +443,15 @@ class SenderSina(QThread):
         self.completed = True
         self.emit(SIGNAL("finished(bool)"),self.completed)
         self.logger.info("sending completed!")
+        
+    def add_follower(self, screen_name):
+        #print "add_follower"
+        param={}
+        param["screen_name"]=screen_name
+        self.api.create_friendship(**param)
+    def delete_follower(self, screen_name):
+        #print "add_follower"
+        param={}
+        param["screen_name"]=screen_name
+        self.api.destroy_friendship(**param)
 
