@@ -128,8 +128,11 @@ class MaoboApp(QApplication):
         Utils.settings=self.controller.get_settings()
         
         self.currentAccount=None
+        self.accountsManaForm=None
+        self.newAccountForm=None
 
         self.commentid_seed = 0
+    
         self.create_accounts_window()
         if len(self.accounts)==0:
             self.create_default_window()
@@ -137,34 +140,34 @@ class MaoboApp(QApplication):
         else:
             self.accountsWin.show()
 
-        #print str(time.time()) + ": " + "MaoboApp __init__ 1"  
-        
-        self.sendersina_service=SenderSina(self.show_notice)
-        self.senderqq_service=SenderQQ(self.show_notice)
+            #print str(time.time()) + ": " + "MaoboApp __init__ 1"  
+            
+            self.sendersina_service=SenderSina(self.show_notice)
+            self.senderqq_service=SenderQQ(self.show_notice)
 
-        #self.clean_temp_images("/home/user/.sharing/outbox")
+            #self.clean_temp_images("/home/user/.sharing/outbox")
 
 
-        print str(time.time()) + ": " + "MaoboApp __init__ 2"
-                 
-        Utils.g_orientation=self.get_current_orientation()
-        #print Utils.g_orietation
-        self.create_timeline_window()
-        self.create_status_window()
-        self.newStatusWin=None
-        self.accountsManaForm=None
-        self.newAccountForm=None
-        self.userWin = None
+            print str(time.time()) + ": " + "MaoboApp __init__ 2"
+            
+            Utils.g_orientation=self.get_current_orientation()
+            #print Utils.g_orietation
+            self.create_timeline_window()
+            self.create_status_window()
+            self.newStatusWin=None
+            self.userWin = None
 
-        self.init_unread_updaters()
-        
-        print "first threading start"
-        self.sm_auto_update_unread = _SM_AUTO_IDLE
-        self.t = threading.Timer(5.0, self.cb_update_unread_counts)
-        self.t.start()
-        print str(time.time()) + ": " + "MaoboApp __init__ 3"
-        
-        self.connect(self, SIGNAL("lastWindowClosed()"), self.lastWindowClosed)
+            self.init_unread_updaters()
+            
+            print "first threading start"
+            self.sm_auto_update_unread = _SM_AUTO_IDLE
+            self.t = threading.Timer(5.0, self.cb_update_unread_counts)
+            self.t.start()
+            print str(time.time()) + ": " + "MaoboApp __init__ 3"
+            
+            self.connect(self, SIGNAL("lastWindowClosed()"), self.lastWindowClosed)
+            
+        self.doReloadBackgroundIcon()
           
     def lastWindowClosed(self):
         print "lastWindowClosed"        
@@ -1385,9 +1388,27 @@ class MaoboApp(QApplication):
                 #self.show_notice("成功保存微博设置")
         if self.accountsWin != None:
             self.accountsWin.set_running_mode(Utils.settings["background"])
+            # change background icon show or hide.
+            self.doReloadBackgroundIcon()
+                
 
         self.settingForm.hide()
 
+    def doReloadBackgroundIcon(self):
+        print Utils.settings["background"]
+        if Utils.settings["background"] == "True":
+            print "show background icon"
+            try:                    
+                os.system('cp /opt/maebo/maebo_status_menu.desktop /usr/share/applications/hildon-status-menu/')
+            except:
+                pass
+        else:
+            print "hide background icon"
+            try:
+                os.system('rm /usr/share/applications/hildon-status-menu/maebo_status_menu.desktop')
+            except:
+                pass
+    
     def show_about_form(self):
         aboutForm=AboutForm(self.accountsWin)
         aboutForm.show()
